(defconst my-auto-compile-cache-directory "~/.emacs.d/.gen/byte-code-cache")

(when
    (file-exists-p (expand-file-name "delete" my-auto-compile-cache-directory))
  (dolist (file (directory-files my-auto-compile-cache-directory nil "^[^.]"))
    (delete-file (expand-file-name file my-auto-compile-cache-directory))))
(make-directory my-auto-compile-cache-directory t)

(defvar my-auto-compile-disable nil)

(defvar my-auto-compile-original-load-source-file-function
  load-source-file-function)
(defvar my-auto-compile-inner-load-files nil)
(defvar my-auto-compile-inner nil)

;; 06/2012: sometimes this form is not properly executed
(defun my-advice-byte-compile-log-file ()
  (defun byte-compile-log-file ()
    (with-current-buffer (get-buffer-create byte-compile-log-buffer)
      ;; there is no other obvious place to create the buffer and set the mode
      (compilation-mode))))
(my-advice-byte-compile-log-file)

(defun my-auto-compile-disable ()
  (or my-auto-compile-disable (not (equal emacs-version "24.1.1"))))

(defun my-auto-compile-file (fullname file &optional noerror nomessage no-load)
  (let* ((byte-file)
	 (action
	  (cond
	   ((or (my-auto-compile-disable)
		(not (string-match-p
		      (concat "^~/\\(\\.emacs\\.d/elisp\\|"
			      "tracked/projects\\)/")
		      (abbreviate-file-name fullname))))
	    nil)
	   ((progn
	      (setq byte-file
		    (expand-file-name
		     (concat (file-name-sans-extension
			      (file-name-nondirectory fullname))
			     "-"
			     (substring (sha1 fullname) 0 2)
			     ".elc")
		     my-auto-compile-cache-directory))
	      (not (or (not (file-exists-p byte-file))
		       (file-newer-than-file-p file byte-file))))
	    t)
	   (my-auto-compile-inner
	    (add-to-list 'my-auto-compile-inner-load-files fullname)
	    nil)
	   (t
	    (require 'bytecomp)
	    (my-advice-byte-compile-log-file)
	    (let ((byte-compile-dest-file-function (lambda (_) byte-file))
		  (my-auto-compile-inner t))
	      (when (byte-compile-file file (not no-load))
		(unless (eq system-type 'windows-nt)
		  (let ((link (replace-regexp-in-string "c$" "" byte-file)))
		    (unless (file-exists-p link)
		      (start-process "symlink" nil
				     "ln" "--symbolic"
				     fullname (expand-file-name link)))))
		'nothing))))))
    (cond ((or no-load (eq action 'nothing)) t)
	  (action (load byte-file noerror nomessage t))
	  (t (funcall my-auto-compile-original-load-source-file-function
		      fullname file noerror nomessage)))))

(defun my-load-source-file-function (fullname file &optional noerror nomessage)
  (let ((result (my-auto-compile-file fullname file noerror nomessage)))
    (unless my-auto-compile-inner
      (while my-auto-compile-inner-load-files
	(let ((file (pop my-auto-compile-inner-load-files)))
	  (my-auto-compile-file file file nil nil t))))
    result))

(setq load-source-file-function 'my-load-source-file-function)
